@@ -177,11 +177,11 @@ dmp_set_size(XfcePanelPlugin *plugin, gint wsize, DMPlugin *dmp)
         
         gtk_widget_size_request(dmp->label, &req);
         if(orientation == GTK_ORIENTATION_HORIZONTAL)
-            width += req.width + BORDER/2;
+            width += req.width + BORDER / 2;
         else {
             width = (width > req.width ? width : req.width
                      + GTK_WIDGET(dmp->label)->style->xthickness);
-            height += req.height + BORDER/2;
+            height += req.height + BORDER / 2;
         }
     }
     
@@ -213,9 +213,9 @@ dmp_set_orientation(XfcePanelPlugin *plugin,
             gtk_bin_get_child(GTK_BIN(dmp->button)));
     
     if(xfce_panel_plugin_get_orientation(plugin) == GTK_ORIENTATION_HORIZONTAL)
-        dmp->box = gtk_hbox_new(FALSE, BORDER/2);
+        dmp->box = gtk_hbox_new(FALSE, BORDER / 2);
     else
-        dmp->box = gtk_vbox_new(FALSE, BORDER/2);
+        dmp->box = gtk_vbox_new(FALSE, BORDER / 2);
     gtk_container_set_border_width(GTK_CONTAINER(dmp->box), 0);
     gtk_widget_show(dmp->box);
     gtk_container_add(GTK_CONTAINER(dmp->button), dmp->box);
@@ -341,7 +341,7 @@ menu_deactivated(GtkWidget *menu, gpointer user_data)
 }
 
 static void
-menu_activate(DMPlugin *dmp)
+menu_activate(DMPlugin *dmp, gboolean at_pointer)
 {
     GtkWidget *menu;
     GtkWidget *button;
@@ -362,11 +362,12 @@ menu_activate(DMPlugin *dmp)
         id = g_signal_connect(menu, "deactivate", 
                 G_CALLBACK(menu_deactivated), dmp);
         g_object_set_data(G_OBJECT(menu), "sig_id", GUINT_TO_POINTER(id));
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+        if (!at_pointer)
+          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
         xfce_panel_plugin_register_menu (dmp->plugin, GTK_MENU(menu));
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
-                       (GtkMenuPositionFunc)dmp_position_menu, dmp,
-                       1, gtk_get_current_event_time());
+                       (GtkMenuPositionFunc)(at_pointer ? NULL : dmp_position_menu),
+                       dmp, 1, gtk_get_current_event_time());
     } else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
 }
@@ -392,7 +393,11 @@ dmp_message_received(GtkWidget *w,
 
     if(evt->data_format == 8) {
         if(strcmp(XFCE_MENU_MESSAGE, evt->data.b) == 0) {
-            menu_activate(dmp);
+            menu_activate(dmp, FALSE);
+            return TRUE;
+        }
+        if(strcmp(XFCE_MENU_AT_POINTER_MESSAGE, evt->data.b) == 0) {
+            menu_activate(dmp, TRUE);
             return TRUE;
         }
     }
@@ -415,7 +420,7 @@ dmp_popup(GtkWidget *w,
         return FALSE;
     }
 
-    menu_activate(dmp);
+    menu_activate(dmp, FALSE);
 
     return TRUE;
 }
@@ -709,20 +714,30 @@ dmp_edit_menu_clicked_cb(GtkWidget *w, gpointer user_data)
 {
     DMPlugin *dmp = user_data;
     GError *err = NULL;
-    const gchar *menu_file;
+    const gchar *menu_file = NULL;
     gchar cmd[PATH_MAX];
     
     g_return_if_fail(dmp && dmp->desktop_menu);
     
-    menu_file = xfce_desktop_menu_get_menu_file(dmp->desktop_menu);
-    if(!menu_file)
-        return;
     
-    g_snprintf(cmd, PATH_MAX, "%s/xfce4-menueditor \"%s\"", BINDIR, menu_file);
+    if(dmp->use_default_menu)
+        g_snprintf(cmd, PATH_MAX, "%s/xfce4-menueditor", BINDIR);
+    else {
+        menu_file = xfce_desktop_menu_get_menu_file(dmp->desktop_menu);
+        if(!menu_file)
+            return;
+        g_snprintf(cmd, PATH_MAX, "%s/xfce4-menueditor \"%s\"", BINDIR,
+                   menu_file);
+    }
+    
     if(xfce_exec(cmd, FALSE, FALSE, NULL))
         return;
     
-    g_snprintf(cmd, PATH_MAX, "xfce4-menueditor \"%s\"", menu_file);
+    if(dmp->use_default_menu)
+        g_strlcpy(cmd, "xfce4-menueditor", PATH_MAX);
+    else
+        g_snprintf(cmd, PATH_MAX, "xfce4-menueditor \"%s\"", menu_file);
+    
     if(!xfce_exec(cmd, FALSE, FALSE, &err)) {
         xfce_warn(_("Unable to launch xfce4-menueditor: %s"), err->message);
         g_error_free(err);
@@ -740,28 +755,22 @@ dmp_options_dlg_response_cb(GtkDialog *dialog, gint response, DMPlugin *dmp)
 static void
 dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
 {
-    GtkWidget *dlg, *header,*topvbox, *vbox, *hbox, *frame, *frame_bin, *spacer;
+    GtkWidget *dlg, *topvbox, *vbox, *hbox, *frame, *frame_bin, *spacer;
     GtkWidget *label, *image, *filebutton, *chk, *radio, *entry, *btn;
     
     xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
     
     xfce_panel_plugin_block_menu(plugin);
     
-    dlg = gtk_dialog_new_with_buttons(_("Edit Properties"),
+    dlg = xfce_titled_dialog_new_with_buttons(_("Xfce Menu"),
                         GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(plugin))),
                         GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
                         GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
-    gtk_container_set_border_width(GTK_CONTAINER(dlg), BORDER);
+    gtk_container_set_border_width(GTK_CONTAINER(dlg), 0);
     g_signal_connect(G_OBJECT(dlg), "response",
                      G_CALLBACK(dmp_options_dlg_response_cb), dmp);
     
-    header = xfce_create_header(NULL, _("Xfce Menu"));
-    gtk_widget_set_size_request(GTK_BIN(header)->child, -1, 32);  /* iconless size hack */
-    gtk_container_set_border_width(GTK_CONTAINER(header), BORDER/2);
-    gtk_widget_show(header);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), header, FALSE, TRUE, 0);
-    
-    topvbox = gtk_vbox_new(FALSE, BORDER/2);
+    topvbox = gtk_vbox_new(FALSE, BORDER / 2);
     gtk_widget_show(topvbox);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), topvbox, TRUE, TRUE, 0);
     
@@ -769,12 +778,12 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
-    vbox = gtk_vbox_new(FALSE, BORDER/2);
+    vbox = gtk_vbox_new(FALSE, BORDER / 2);
     gtk_widget_show(vbox);
     gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
     
-    hbox = gtk_hbox_new(FALSE, BORDER/2);
-    gtk_container_set_border_width(GTK_CONTAINER(hbox), BORDER/2);
+    hbox = gtk_hbox_new(FALSE, BORDER / 2);
+    gtk_container_set_border_width(GTK_CONTAINER(hbox), BORDER / 2);
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
@@ -802,12 +811,12 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
-    vbox = gtk_vbox_new(FALSE, BORDER/2);
+    vbox = gtk_vbox_new(FALSE, BORDER / 2);
     gtk_widget_show(vbox);
     gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
     
     /* 2nd radio button's child hbox */
-    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    hbox = gtk_hbox_new(FALSE, BORDER / 2);
     gtk_widget_show(hbox);
     
     radio = gtk_radio_button_new_with_mnemonic(NULL, _("Use default _desktop menu file"));
@@ -866,7 +875,7 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     gtk_box_pack_start(GTK_BOX(vbox), spacer, FALSE, FALSE, 0);
     gtk_widget_set_size_request(spacer, -1, 4);
     
-    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    hbox = gtk_hbox_new(FALSE, BORDER / 2);
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
@@ -880,11 +889,11 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
-    vbox = gtk_vbox_new(FALSE, BORDER/2);
+    vbox = gtk_vbox_new(FALSE, BORDER / 2);
     gtk_widget_show(vbox);
     gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
     
-    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    hbox = gtk_hbox_new(FALSE, BORDER / 2);
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
@@ -915,7 +924,7 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     dmp->icons_chk = chk = gtk_check_button_new_with_mnemonic(_("Show _icons in menu"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk), dmp->show_menu_icons);
     gtk_widget_show(chk);
-    gtk_box_pack_start(GTK_BOX(vbox), chk, FALSE, FALSE, BORDER/2);
+    gtk_box_pack_start(GTK_BOX(vbox), chk, FALSE, FALSE, BORDER / 2);
     g_signal_connect(G_OBJECT(chk), "toggled", G_CALLBACK(icon_chk_cb), dmp);
     
     gtk_widget_show(dlg);
@@ -993,9 +1002,9 @@ dmp_new(XfcePanelPlugin *plugin)
     gtk_tooltips_set_tip(dmp->tooltip, dmp->button, dmp->button_title, NULL);
     
     if(xfce_panel_plugin_get_orientation(plugin) == GTK_ORIENTATION_HORIZONTAL)
-        dmp->box = gtk_hbox_new(FALSE, BORDER/2);
+        dmp->box = gtk_hbox_new(FALSE, BORDER / 2);
     else
-        dmp->box = gtk_vbox_new(FALSE, BORDER/2);
+        dmp->box = gtk_vbox_new(FALSE, BORDER / 2);
     gtk_container_set_border_width(GTK_CONTAINER(dmp->box), 0);
     gtk_widget_show(dmp->box);
     gtk_container_add(GTK_CONTAINER(dmp->button), dmp->box);
